@@ -15,33 +15,44 @@
 #include "soup-connection.h"
 #include "soup-server-message-private.h"
 #include "soup-message-headers-private.h"
+#include "soup-socket.h"
 #include "soup-uri-utils-private.h"
 
+/**
+ * SECTION:soup-server-message
+ * @short_description: An HTTP server request and response.
+ * @see_also: #SoupMessageHeaders, #SoupMessageBody
+ *
+ * A SoupServerMessage represents an HTTP message that is being sent or
+ * received on a #SoupServer
+ *
+ * #SoupServer will create #SoupServerMessage<!-- -->s automatically for
+ * incoming requests, which your application will receive via handlers.
+ *
+ * Note that libsoup's terminology here does not quite match the HTTP
+ * specification: in RFC 2616, an "HTTP-message" is
+ * <emphasis>either</emphasis> a Request, <emphasis>or</emphasis> a
+ * Response. In libsoup, a #SoupServerMessage combines both the request and
+ * the response.
+ **/
+
 /**
  * SoupServerMessage:
  *
- * An HTTP server request and response pair.
- *
- * A SoupServerMessage represents an HTTP message that is being sent or
- * received on a [class@Server].
- *
- * [class@Server] will create `SoupServerMessage`s automatically for
- * incoming requests, which your application will receive via handlers.
- *
- * Note that libsoup's terminology here does not quite match the HTTP
- * specification: in RFC 2616, an "HTTP-message" is *either* a Request, *or* a
- * Response. In libsoup, a #SoupServerMessage combines both the request and the
- * response.
- **/
+ * Class represnting an HTTP request and response pair for a server.
+ */
 
 struct _SoupServerMessage {
         GObject             parent;
 
-        SoupServerConnection *conn;
+        SoupSocket         *sock;
+        GSocket            *gsock;
         SoupAuthDomain     *auth_domain;
         char               *auth_user;
 
+        GSocketAddress     *remote_addr;
         char               *remote_ip;
+        GSocketAddress     *local_addr;
 
         const char         *method;
         SoupHTTPVersion     http_version;
@@ -58,12 +69,9 @@ struct _SoupServerMessage {
         SoupMessageBody    *response_body;
         SoupMessageHeaders *response_headers;
 
-        SoupServerMessageIO *io_data;
+        SoupServerMessageIOData *io_data;
 
         gboolean                 options_ping;
-
-        GTlsCertificate      *tls_peer_certificate;
-        GTlsCertificateFlags  tls_peer_certificate_errors;
 };
 
 struct _SoupServerMessageClass {
@@ -83,7 +91,6 @@ enum {
         GOT_CHUNK,
         GOT_BODY,
 
-        CONNECTED,
         DISCONNECTED,
         FINISHED,
 
@@ -94,17 +101,6 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-enum {
-	PROP_0,
-
-	PROP_TLS_PEER_CERTIFICATE,
-	PROP_TLS_PEER_CERTIFICATE_ERRORS,
-
-	LAST_PROPERTY
-};
-
-static GParamSpec *properties[LAST_PROPERTY] = { NULL, };
-
 static void
 soup_server_message_init (SoupServerMessage *msg)
 {
@@ -120,13 +116,18 @@ soup_server_message_finalize (GObject *object)
 {
         SoupServerMessage *msg = SOUP_SERVER_MESSAGE (object);
 
+        soup_server_message_io_data_free (msg->io_data);
+
         g_clear_object (&msg->auth_domain);
         g_clear_pointer (&msg->auth_user, g_free);
+        g_clear_object (&msg->remote_addr);
+        g_clear_object (&msg->local_addr);
 
-        if (msg->conn) {
-                g_signal_handlers_disconnect_by_data (msg->conn, msg);
-                g_object_unref (msg->conn);
+        if (msg->sock) {
+                g_signal_handlers_disconnect_by_data (msg->sock, msg);
+                g_object_unref (msg->sock);
         }
+        g_clear_object (&msg->gsock);
         g_clear_pointer (&msg->remote_ip, g_free);
 
         g_clear_pointer (&msg->uri, g_uri_unref);
@@ -140,32 +141,12 @@ soup_server_message_finalize (GObject *object)
         G_OBJECT_CLASS (soup_server_message_parent_class)->finalize (object);
 }
 
-static void
-soup_server_message_get_property (GObject *object, guint prop_id,
-                                  GValue *value, GParamSpec *pspec)
-{
-	SoupServerMessage *msg = SOUP_SERVER_MESSAGE (object);
-
-	switch (prop_id) {
-	case PROP_TLS_PEER_CERTIFICATE:
-		g_value_set_object (value, msg->tls_peer_certificate);
-		break;
-	case PROP_TLS_PEER_CERTIFICATE_ERRORS:
-		g_value_set_flags (value, msg->tls_peer_certificate_errors);
-		break;
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
 static void
 soup_server_message_class_init (SoupServerMessageClass *klass)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
         object_class->finalize = soup_server_message_finalize;
-        object_class->get_property = soup_server_message_get_property;
 
         /**
          * SoupServerMessage::wrote-informational:
@@ -205,11 +186,11 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
          * Emitted immediately after writing a body chunk for a message.
          *
          * Note that this signal is not parallel to
-         * [signal@ServerMessage::got-chunk]; it is emitted only when a complete
-         * chunk (added with [method@MessageBody.append] or
-         * [method@MessageBody.append_bytes] has been written. To get
+         * #SoupServerMessage::got-chunk; it is emitted only when a complete
+         * chunk (added with soup_message_body_append() or
+         * soup_message_body_append_bytes()) has been written. To get
          * more useful continuous progress information, use
-         * [signal@ServerMessage::wrote-body-data].
+         * #SoupServerMessage::wrote-body-data.
          */
         signals[WROTE_CHUNK] =
                 g_signal_new ("wrote-chunk",
@@ -274,10 +255,10 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
          * @msg: the message
          * @chunk: the just-read chunk
          *
-         * Emitted after receiving a chunk of a message body.
-         *
-         * Note that "chunk" in this context means any subpiece of the body, not
-         * necessarily the specific HTTP 1.1 chunks sent by the other side.
+         * Emitted after receiving a chunk of a message body. Note
+         * that "chunk" in this context means any subpiece of the
+         * body, not necessarily the specific HTTP 1.1 chunks sent by
+         * the other side.
          */
         signals[GOT_CHUNK] =
                 g_signal_new ("got-chunk",
@@ -309,7 +290,7 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
          * @msg: the message
          *
          * Emitted when all HTTP processing is finished for a message.
-         * (After [signal@ServerMessage::wrote-body]).
+         * (After #SoupServerMessage::wrote-body).
          */
         signals[FINISHED] =
                 g_signal_new ("finished",
@@ -319,20 +300,6 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
                               NULL, NULL,
                               NULL,
                               G_TYPE_NONE, 0);
-        /**
-         * SoupServerMessage::connected:
-         * @msg: the message
-         *
-         * Emitted when the @msg's socket is connected and the TLS handshake completed.
-         */
-        signals[CONNECTED] =
-                g_signal_new ("connected",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              0,
-                              NULL, NULL,
-                              NULL,
-                              G_TYPE_NONE, 0);
 
         /**
          * SoupServerMessage::disconnected:
@@ -361,8 +328,8 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
 	 * @tls_errors.
 	 *
 	 * Returns: %TRUE to accept the TLS certificate and stop other
-	 *   handlers from being invoked, or %FALSE to propagate the
-	 *   event further.
+	 *     handlers from being invoked, or %FALSE to propagate the
+	 *     event further.
 	 */
 	signals[ACCEPT_CERTIFICATE] =
 		g_signal_new ("accept-certificate",
@@ -374,59 +341,18 @@ soup_server_message_class_init (SoupServerMessageClass *klass)
 			      G_TYPE_BOOLEAN, 2,
 			      G_TYPE_TLS_CERTIFICATE,
 			      G_TYPE_TLS_CERTIFICATE_FLAGS);
-
-	/**
-	 * SoupServerMessage:tls-peer-certificate:
-	 *
-	 * The peer's #GTlsCertificate associated with the message
-	 *
-	 * Since: 3.2
-	 */
-        properties[PROP_TLS_PEER_CERTIFICATE] =
-		g_param_spec_object ("tls-peer-certificate",
-				     "TLS Peer Certificate",
-				     "The TLS peer certificate associated with the message",
-				     G_TYPE_TLS_CERTIFICATE,
-				     G_PARAM_READABLE |
-				     G_PARAM_STATIC_STRINGS);
-
-	/**
-	 * SoupServerMessage:tls-peer-certificate-errors:
-	 *
-	 * The verification errors on #SoupServerMessage:tls-peer-certificate
-	 *
-	 * Since: 3.2
-	 */
-        properties[PROP_TLS_PEER_CERTIFICATE_ERRORS] =
-		g_param_spec_flags ("tls-peer-certificate-errors",
-				    "TLS Peer Certificate Errors",
-				    "The verification errors on the message's TLS peer certificate",
-				    G_TYPE_TLS_CERTIFICATE_FLAGS, 0,
-				    G_PARAM_READABLE |
-				    G_PARAM_STATIC_STRINGS);
-
-        g_object_class_install_properties (object_class, LAST_PROPERTY, properties);
 }
 
 static void
-connection_connected (SoupServerMessage *msg)
+socket_disconnected (SoupServerMessage *msg)
 {
-        g_assert (!msg->io_data);
-        msg->io_data = soup_server_connection_get_io_data (msg->conn);
-        g_signal_emit (msg, signals[CONNECTED], 0);
-}
-
-static void
-connection_disconnected (SoupServerMessage *msg)
-{
-        msg->io_data = NULL;
         g_signal_emit (msg, signals[DISCONNECTED], 0);
 }
 
 static gboolean
-connection_accept_certificate (SoupServerMessage    *msg,
-                               GTlsCertificate      *tls_certificate,
-                               GTlsCertificateFlags *tls_errors)
+socket_accept_certificate (SoupServerMessage    *msg,
+                           GTlsCertificate      *tls_certificate,
+                           GTlsCertificateFlags *tls_errors)
 {
 	gboolean accept = FALSE;
 
@@ -435,51 +361,22 @@ connection_accept_certificate (SoupServerMessage    *msg,
 	return accept;
 }
 
-static void
-soup_server_message_set_tls_peer_certificate (SoupServerMessage   *msg,
-                                              GTlsCertificate     *tls_certificate,
-                                              GTlsCertificateFlags tls_errors)
-{
-        if (msg->tls_peer_certificate == tls_certificate && msg->tls_peer_certificate_errors == tls_errors)
-                return;
-
-        g_clear_object (&msg->tls_peer_certificate);
-        msg->tls_peer_certificate = tls_certificate ? g_object_ref (tls_certificate) : NULL;
-        msg->tls_peer_certificate_errors = tls_errors;
-        g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_TLS_PEER_CERTIFICATE]);
-        g_object_notify_by_pspec (G_OBJECT (msg), properties[PROP_TLS_PEER_CERTIFICATE_ERRORS]);
-}
-
-static void
-re_emit_tls_certificate_changed (SoupServerMessage    *msg,
-                                 GParamSpec           *pspec,
-                                 SoupServerConnection *conn)
-{
-        soup_server_message_set_tls_peer_certificate (msg,
-                                                      soup_server_connection_get_tls_peer_certificate (conn),
-                                                      soup_server_connection_get_tls_peer_certificate_errors (conn));
-}
-
 SoupServerMessage *
-soup_server_message_new (SoupServerConnection *conn)
+soup_server_message_new (SoupSocket *sock)
 {
         SoupServerMessage *msg;
 
         msg = g_object_new (SOUP_TYPE_SERVER_MESSAGE, NULL);
-        msg->conn = g_object_ref (conn);
-        msg->io_data = soup_server_connection_get_io_data (msg->conn);
+        msg->sock = g_object_ref (sock);
+        msg->gsock = soup_socket_get_gsocket (sock);
+        if (msg->gsock)
+                g_object_ref (msg->gsock);
 
-        g_signal_connect_object (conn, "connected",
-                                 G_CALLBACK (connection_connected),
+        g_signal_connect_object (sock, "disconnected",
+                                 G_CALLBACK (socket_disconnected),
                                  msg, G_CONNECT_SWAPPED);
-        g_signal_connect_object (conn, "disconnected",
-                                 G_CALLBACK (connection_disconnected),
-                                 msg, G_CONNECT_SWAPPED);
-        g_signal_connect_object (conn, "accept-certificate",
-                                 G_CALLBACK (connection_accept_certificate),
-                                 msg, G_CONNECT_SWAPPED);
-        g_signal_connect_object (conn, "notify::tls-certificate",
-                                 G_CALLBACK (re_emit_tls_certificate_changed),
+        g_signal_connect_object (sock, "accept-certificate",
+                                 G_CALLBACK (socket_accept_certificate),
                                  msg, G_CONNECT_SWAPPED);
 
         return msg;
@@ -494,10 +391,10 @@ soup_server_message_set_uri (SoupServerMessage *msg,
         msg->uri = soup_uri_copy_with_normalized_flags (uri);
 }
 
-SoupServerConnection *
-soup_server_message_get_connection (SoupServerMessage *msg)
+SoupSocket *
+soup_server_message_get_soup_socket (SoupServerMessage *msg)
 {
-        return msg->conn;
+        return msg->sock;
 }
 
 void
@@ -517,9 +414,6 @@ soup_server_message_set_auth (SoupServerMessage *msg,
 gboolean
 soup_server_message_is_keepalive (SoupServerMessage *msg)
 {
-        if (msg->http_version == SOUP_HTTP_2_0)
-                return TRUE;
-
         if (msg->status_code == SOUP_STATUS_OK && msg->method == SOUP_METHOD_CONNECT)
                 return TRUE;
 
@@ -555,76 +449,19 @@ soup_server_message_is_keepalive (SoupServerMessage *msg)
 }
 
 void
-soup_server_message_read_request (SoupServerMessage        *msg,
-                                  SoupMessageIOCompletionFn completion_cb,
-                                  gpointer                  user_data)
+soup_server_message_set_io_data (SoupServerMessage       *msg,
+                                 SoupServerMessageIOData *io)
 {
-        soup_server_message_io_read_request (msg->io_data, msg, completion_cb, user_data);
+        soup_server_message_io_data_free (msg->io_data);
+        msg->io_data = io;
 }
 
-SoupServerMessageIO *
+SoupServerMessageIOData *
 soup_server_message_get_io_data (SoupServerMessage *msg)
 {
         return msg->io_data;
 }
 
-/**
- * soup_server_message_pause:
- * @msg: a SoupServerMessage
- *
- * Pauses I/O on @msg.
- *
- * This can be used when you need to return from the server handler without
- * having the full response ready yet. Use [method@ServerMessage.unpause] to
- * resume I/O.
- *
- * Since: 3.2
- */
-void
-soup_server_message_pause (SoupServerMessage *msg)
-{
-        g_return_if_fail (SOUP_IS_SERVER_MESSAGE (msg));
-        g_return_if_fail (msg->io_data != NULL);
-
-        soup_server_message_io_pause (msg->io_data, msg);
-}
-
-/**
- * soup_server_message_unpause:
- * @msg: a SoupServerMessage
- *
- * Resumes I/O on @msg.
- *
- * Use this to resume after calling [method@ServerMessage.pause], or after
- * adding a new chunk to a chunked response. I/O won't actually resume until you
- * return to the main loop.
- *
- * Since: 3.2
- */
-void
-soup_server_message_unpause (SoupServerMessage *msg)
-{
-        g_return_if_fail (SOUP_IS_SERVER_MESSAGE (msg));
-        g_return_if_fail (msg->io_data != NULL);
-
-        soup_server_message_io_unpause (msg->io_data, msg);
-}
-
-gboolean
-soup_server_message_is_io_paused (SoupServerMessage *msg)
-{
-        return msg->io_data && soup_server_message_io_is_paused (msg->io_data, msg);
-}
-
-void
-soup_server_message_finish (SoupServerMessage *msg)
-{
-        if (!msg->io_data)
-                return;
-
-        soup_server_message_io_finished (g_steal_pointer (&msg->io_data), msg);
-}
-
 void
 soup_server_message_cleanup_response (SoupServerMessage *msg)
 {
@@ -843,9 +680,9 @@ soup_server_message_set_http_version (SoupServerMessage *msg,
  * soup_server_message_get_reason_phrase:
  * @msg: a #SoupServerMessage:
  *
- * Get the HTTP reason phrase of @msg.
+ * Get the HTTP reason phrase of @msg or %NULL.
  *
- * Returns: (nullable): the reason phrase.
+ * Returns: the reason phrase.
  */
 const char *
 soup_server_message_get_reason_phrase (SoupServerMessage *msg)
@@ -877,10 +714,9 @@ soup_server_message_get_status (SoupServerMessage *msg)
  * @status_code: an HTTP status code
  * @reason_phrase: (nullable): a reason phrase
  *
- * Sets @msg's status code to @status_code.
- *
- * If @status_code is a known value and @reason_phrase is %NULL, the
- * reason_phrase will be set automatically.
+ * Sets @msg's status code to @status_code. If @status_code is a
+ * known value and @reason_phrase is %NULL, the reason_phrase will
+ * be set automatically.
  **/
 void
 soup_server_message_set_status (SoupServerMessage *msg,
@@ -956,13 +792,13 @@ soup_server_message_set_response (SoupServerMessage *msg,
  * @redirect_uri: the URI to redirect @msg to
  *
  * Sets @msg's status_code to @status_code and adds a Location header
- * pointing to @redirect_uri. Use this from a [class@Server] when you
+ * pointing to @redirect_uri. Use this from a #SoupServer when you
  * want to redirect the client to another URI.
  *
  * @redirect_uri can be a relative URI, in which case it is
  * interpreted relative to @msg's current URI. In particular, if
  * @redirect_uri is just a path, it will replace the path
- * *and query* of @msg's URI.
+ * <emphasis>and query</emphasis> of @msg's URI.
  */
 void
 soup_server_message_set_redirect (SoupServerMessage *msg,
@@ -989,7 +825,7 @@ soup_server_message_set_redirect (SoupServerMessage *msg,
  * soup_server_message_get_socket:
  * @msg: a #SoupServerMessage
  *
- * Retrieves the [class@Gio.Socket] that @msg is associated with.
+ * Retrieves the #GSocket that @msg is associated with.
  *
  * If you are using this method to observe when multiple requests are
  * made on the same persistent HTTP connection (eg, as the ntlm-test
@@ -999,52 +835,66 @@ soup_server_message_set_redirect (SoupServerMessage *msg,
  * previously-destroyed socket to represent a new socket.
  *
  * Returns: (nullable) (transfer none): the #GSocket that @msg is
- *   associated with, %NULL if you used [method@Server.accept_iostream].
+ * associated with, %NULL if you used soup_server_accept_iostream().
  */
 GSocket *
 soup_server_message_get_socket (SoupServerMessage *msg)
 {
         g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), NULL);
 
-        return soup_server_connection_get_socket (msg->conn);
+        return msg->gsock;
 }
 
 /**
  * soup_server_message_get_remote_address:
  * @msg: a #SoupServerMessage
  *
- * Retrieves the [class@Gio.SocketAddress] associated with the remote end
+ * Retrieves the #GSocketAddress associated with the remote end
  * of a connection.
  *
  * Returns: (nullable) (transfer none): the #GSocketAddress
- *   associated with the remote end of a connection, it may be
- *   %NULL if you used [class@Server.accept_iostream].
+ *     associated with the remote end of a connection, it may be
+ *     %NULL if you used soup_server_accept_iostream().
  */
 GSocketAddress *
 soup_server_message_get_remote_address (SoupServerMessage *msg)
 {
         g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), NULL);
 
-        return soup_server_connection_get_remote_address (msg->conn);
+        if (msg->remote_addr)
+                return msg->remote_addr;
+
+        msg->remote_addr = msg->gsock ?
+                g_socket_get_remote_address (msg->gsock, NULL) :
+                G_SOCKET_ADDRESS (g_object_ref (soup_socket_get_remote_address (msg->sock)));
+
+        return msg->remote_addr;
 }
 
 /**
  * soup_server_message_get_local_address:
  * @msg: a #SoupServerMessage
  *
- * Retrieves the [class@Gio.SocketAddress] associated with the local end
+ * Retrieves the #GSocketAddress associated with the local end
  * of a connection.
  *
  * Returns: (nullable) (transfer none): the #GSocketAddress
- *   associated with the local end of a connection, it may be
- *   %NULL if you used [method@Server.accept_iostream].
+ *     associated with the local end of a connection, it may be
+ *     %NULL if you used soup_server_accept_iostream().
  */
 GSocketAddress *
 soup_server_message_get_local_address (SoupServerMessage *msg)
 {
         g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), NULL);
 
-        return soup_server_connection_get_local_address (msg->conn);
+        if (msg->local_addr)
+                return msg->local_addr;
+
+        msg->local_addr = msg->gsock ?
+                g_socket_get_local_address (msg->gsock, NULL) :
+                G_SOCKET_ADDRESS (g_object_ref (soup_socket_get_local_address (msg->sock)));
+
+        return msg->local_addr;
 }
 
 /**
@@ -1055,23 +905,29 @@ soup_server_message_get_local_address (SoupServerMessage *msg)
  * connection.
  *
  * Returns: (nullable): the IP address associated with the remote
- *   end of a connection, it may be %NULL if you used
- *   [method@Server.accept_iostream].
+ *     end of a connection, it may be %NULL if you used
+ *     soup_server_accept_iostream().
  */
 const char *
 soup_server_message_get_remote_host (SoupServerMessage *msg)
 {
         g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), NULL);
 
-        if (!msg->remote_ip) {
-                GSocketAddress *addr = soup_server_connection_get_remote_address (msg->conn);
+        if (msg->remote_ip)
+                return msg->remote_ip;
+
+        if (msg->gsock) {
+                GSocketAddress *addr = soup_server_message_get_remote_address (msg);
                 GInetAddress *iaddr;
 
                 if (!addr || !G_IS_INET_SOCKET_ADDRESS (addr))
                         return NULL;
-
                 iaddr = g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (addr));
                 msg->remote_ip = g_inet_address_to_string (iaddr);
+        } else {
+                GInetSocketAddress *addr = G_INET_SOCKET_ADDRESS (soup_socket_get_remote_address (msg->sock));
+                GInetAddress *inet_addr = g_inet_socket_address_get_address (addr);
+                msg->remote_ip = g_inet_address_to_string (inet_addr);
         }
 
         return msg->remote_ip;
@@ -1081,13 +937,12 @@ soup_server_message_get_remote_host (SoupServerMessage *msg)
  * soup_server_message_steal_connection:
  * @msg: a #SoupServerMessage
  *
- * "Steals" the HTTP connection associated with @msg from its #SoupServer. This
- * happens immediately, regardless of the current state of the connection; if
- * the response to @msg has not yet finished being sent, then it will be
- * discarded; you can steal the connection from a
- * [signal@ServerMessage::wrote-informational] or
- * [signal@ServerMessage::wrote-body] signal handler if you need to wait for
- * part or all of the response to be sent.
+ * "Steals" the HTTP connection associated with @msg from its
+ * #SoupServer. This happens immediately, regardless of the current
+ * state of the connection; if the response to @msg has not yet finished
+ * being sent, then it will be discarded; you can steal the connection from a
+ * #SoupServerMessage::wrote-informational or #SoupServerMessage::wrote-body signal
+ * handler if you need to wait for part or all of the response to be sent.
  *
  * Note that when calling this function from C, @msg will most
  * likely be freed as a side effect.
@@ -1103,51 +958,17 @@ soup_server_message_steal_connection (SoupServerMessage *msg)
         GIOStream *stream;
 
         g_object_ref (msg);
-        stream = soup_server_connection_steal (msg->conn);
-        g_signal_handlers_disconnect_by_data (msg, msg->conn);
-        connection_disconnected (msg);
+        stream = soup_server_message_io_steal (msg);
+        if (stream) {
+                g_object_set_data_full (G_OBJECT (stream), "GSocket",
+                                        soup_socket_steal_gsocket (msg->sock),
+                                        g_object_unref);
+        }
+
+        g_signal_handlers_disconnect_by_data (msg, msg->sock);
+
+        socket_disconnected (msg);
         g_object_unref (msg);
 
         return stream;
 }
-
-/**
- * soup_server_message_get_tls_peer_certificate:
- * @msg: a #SoupMessage
- *
- * Gets the peer's #GTlsCertificate associated with @msg's connection.
- * Note that this is not set yet during the emission of
- * SoupServerMessage::accept-certificate signal.
- *
- * Returns: (transfer none) (nullable): @msg's TLS peer certificate,
- *    or %NULL if @msg's connection is not SSL.
- *
- * Since: 3.2
- */
-GTlsCertificate *
-soup_server_message_get_tls_peer_certificate (SoupServerMessage *msg)
-{
-        g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), NULL);
-
-        return msg->tls_peer_certificate;
-}
-
-/**
- * soup_server_message_get_tls_peer_certificate_errors:
- * @msg: a #SoupMessage
- *
- * Gets the errors associated with validating @msg's TLS peer certificate.
- * Note that this is not set yet during the emission of
- * SoupServerMessage::accept-certificate signal.
- *
- * Returns: a #GTlsCertificateFlags with @msg's TLS peer certificate errors.
- *
- * Since: 3.2
- */
-GTlsCertificateFlags
-soup_server_message_get_tls_peer_certificate_errors (SoupServerMessage *msg)
-{
-        g_return_val_if_fail (SOUP_IS_SERVER_MESSAGE (msg), 0);
-
-        return msg->tls_peer_certificate_errors;
-}
