@@ -353,6 +353,21 @@ soup_message_headers_append (SoupMessageHeaders *hdrs,
 		g_hash_table_remove (hdrs->uncommon_concat, header.name);
 }
 
+/*
+ * Appends a header value ensuring that it is valid UTF8.
+ */
+void
+soup_message_headers_append_untrusted_data (SoupMessageHeaders *hdrs,
+                                            const char         *name,
+                                            const char         *value)
+{
+        char *safe_value = g_utf8_make_valid (value, -1);
+        char *safe_name = g_utf8_make_valid (name, -1);
+        soup_message_headers_append (hdrs, safe_name, safe_value);
+        g_free (safe_value);
+        g_free (safe_name);
+}
+
 void
 soup_message_headers_replace_common (SoupMessageHeaders *hdrs,
                                      SoupHeaderName      name,
@@ -1442,26 +1457,37 @@ parse_content_foo (SoupMessageHeaders *hdrs,
 {
 	const char *header;
 	char *semi;
+        char *equal;
 
 	header = soup_message_headers_get_one_common (hdrs, header_name);
 	if (!header)
 		return FALSE;
 
-	if (foo) {
-		*foo = g_strdup (header);
-		semi = strchr (*foo, ';');
-		if (semi) {
-			char *p = semi;
+        /* Some websites send an invalid disposition that only contains parameters;
+         * We can be flexible about handling these by detecting if the first word
+         * is a parameter (foo=bar). */
+        equal = strchr (header, '=');
+        semi = strchr (header, ';');
+        if (header_name == SOUP_HEADER_CONTENT_DISPOSITION &&
+            (equal && (!semi || (equal < semi)))) {
+                semi = (char *)header;
+                if (foo)
+                        *foo = NULL;
+        } else if (foo) {
+                *foo = g_strdup (header);
+                semi = strchr (*foo, ';');
+                if (semi) {
+                        char *p = semi;
 
-			*semi++ = '\0';
-			while (p - 1 > *foo && g_ascii_isspace(p[-1]))
-				*(--p) = '\0';
-		}
-	} else {
-		semi = strchr (header, ';');
-		if (semi)
-			semi++;
-	}
+                        *semi++ = '\0';
+                        while (p - 1 > *foo && g_ascii_isspace(p[-1]))
+                                *(--p) = '\0';
+                }
+        } else {
+                /* Skip type, we don't store it */
+                if (semi)
+                        semi++;
+        }
 
 	if (!params)
 		return TRUE;
@@ -1501,7 +1527,7 @@ set_content_foo (SoupMessageHeaders *hdrs,
 /**
  * soup_message_headers_get_content_type:
  * @hdrs: a #SoupMessageHeaders
- * @params: (out) (element-type utf8 utf8) (allow-none) (transfer full):
+ * @params: (out) (element-type utf8 utf8) (optional) (transfer full):
  *   return location for the Content-Type parameters (eg, "charset"), or
  *   %NULL
  *
@@ -1531,7 +1557,7 @@ soup_message_headers_get_content_type (SoupMessageHeaders  *hdrs,
  * soup_message_headers_set_content_type:
  * @hdrs: a #SoupMessageHeaders
  * @content_type: the MIME type
- * @params: (allow-none) (element-type utf8 utf8): additional
+ * @params: (nullable) (element-type utf8 utf8): additional
  * parameters, or %NULL
  *
  * Sets the "Content-Type" header in @hdrs to @content_type,
@@ -1604,7 +1630,7 @@ soup_message_headers_get_content_disposition (SoupMessageHeaders  *hdrs,
  * soup_message_headers_set_content_disposition:
  * @hdrs: a #SoupMessageHeaders
  * @disposition: the disposition-type
- * @params: (allow-none) (element-type utf8 utf8): additional
+ * @params: (nullable) (element-type utf8 utf8): additional
  * parameters, or %NULL
  *
  * Sets the "Content-Disposition" header in @hdrs to @disposition,
