@@ -73,8 +73,11 @@ do_strictness_test (gconstpointer data)
 	flags = soup_message_get_tls_peer_certificate_errors (msg);
 
         if (test->expected_status == SOUP_STATUS_OK) {
+                const char *ciphersuite_name = soup_message_get_tls_ciphersuite_name (msg);
+                /* Format changed in https://gitlab.gnome.org/GNOME/glib-networking/-/merge_requests/194 */
+                g_assert_true (!g_strcmp0 (ciphersuite_name, "TLS_AES-256-GCM_SHA384")
+                               || !g_strcmp0 (ciphersuite_name, "TLS_AES_256_GCM_SHA384"));
                 g_assert_cmpuint (soup_message_get_tls_protocol_version (msg), ==, G_TLS_PROTOCOL_VERSION_TLS_1_3);
-                g_assert_cmpstr (soup_message_get_tls_ciphersuite_name (msg), ==, "TLS_AES-256-GCM_SHA384");
         } else {
                 g_assert_cmpuint (soup_message_get_tls_protocol_version (msg), ==, G_TLS_PROTOCOL_VERSION_UNKNOWN);
                 g_assert_null (soup_message_get_tls_ciphersuite_name (msg));
